@@ -1,4 +1,3 @@
-
 /*
  * Copyright © 2002 Havoc Pennington
  * Copyright © 2002 Mathias Hasselmann
@@ -46,36 +45,48 @@ struct _TerminalColorScheme
   const GdkRGBA background;
 };
 
+#define COLOR(r, g, b) { .red = (r) / 255.0, .green = (g) / 255.0, .blue = (b) / 255.0, .alpha = 1.0 }
+
 static const TerminalColorScheme color_schemes[] = {
   { N_("Black on light yellow"),
-    { 0, 0, 0, 1 },
-    { 1, 1, 0.866667, 1 }
+    COLOR (0x00, 0x00, 0x00),
+    COLOR (0xff, 0xff, 0xdd)
   },
   { N_("Black on white"),
-    { 0, 0, 0, 1 },
-    { 1, 1, 1, 1 }
+    COLOR (0x00, 0x00, 0x00),
+    COLOR (0xff, 0xff, 0xff)
   },
   { N_("Gray on black"),
-    { 0.666667, 0.666667, 0.666667, 1 },
-    { 0, 0, 0, 1 }
+    COLOR (0xaa, 0xaa, 0xaa),
+    COLOR (0x00, 0x00, 0x00)
   },
   { N_("Green on black"),
-    { 0, 1, 0, 1 },
-    { 0, 0, 0, 1 }
+    COLOR (0x00, 0xff, 0x00),
+    COLOR (0x00, 0x00, 0x00)
   },
   { N_("White on black"),
-    { 1, 1, 1, 1 },
-    { 0, 0, 0, 1 }
+    COLOR (0xff, 0xff, 0xff),
+    COLOR (0x00, 0x00, 0x00)
+  },
+  /* Translators: "Tango" is the name of a colour scheme, "light" can be translated */
+  { N_("Tango light"),
+    COLOR (0x2e, 0x34, 0x36),
+    COLOR (0xee, 0xee, 0xec)
+  },
+  /* Translators: "Tango" is the name of a colour scheme, "dark" can be translated */
+  { N_("Tango dark"),
+    COLOR (0xd3, 0xd7, 0xcf),
+    COLOR (0x2e, 0x34, 0x36)
   },
   /* Translators: "Solarized" is the name of a colour scheme, "light" can be translated */
   { N_("Solarized light"),
-    { 0.396078, 0.482352, 0.513725, 1 },
-    { 0.992156, 0.964705, 0.890196, 1 }
+    COLOR (0x65, 0x7B, 0x83),
+    COLOR (0xfd, 0xf6, 0xe3)
   },
   /* Translators: "Solarized" is the name of a colour scheme, "dark" can be translated */
   { N_("Solarized dark"),
-    { 0.513725, 0.580392, 0.588235, 1 },
-    { 0,        0.168627, 0.211764, 1 }
+    COLOR (0x83, 0x94, 0x96),
+    COLOR (0x00, 0x2b, 0x36)
   },
 };
 
@@ -91,13 +102,11 @@ enum
   TERMINAL_PALETTE_N_BUILTINS
 };
 
-#define COLOR(r, g, b) { .red = (r) / 255.0, .green = (g) / 255.0, .blue = (b) / 255.0, .alpha = 1.0 }
-
 static const GdkRGBA terminal_palettes[TERMINAL_PALETTE_N_BUILTINS][TERMINAL_PALETTE_SIZE] =
 {
   /* Tango palette */
   {
-    COLOR (0x00, 0x00, 0x00),
+    COLOR (0x2e, 0x34, 0x36),
     COLOR (0xcc, 0x00, 0x00),
     COLOR (0x4e, 0x9a, 0x06),
     COLOR (0xc4, 0xa0, 0x00),
@@ -407,7 +416,7 @@ profile_palette_notify_colorpickers_cb (GSettings *profile,
     {
       char name[32];
 
-      g_snprintf (name, sizeof (name), "palette-colorpicker-%" G_GSIZE_FORMAT, i + 1);
+      g_snprintf (name, sizeof (name), "palette-colorpicker-%" G_GSIZE_FORMAT, i);
       w = (GtkWidget *) gtk_builder_get_object  (builder, name);
 
       g_signal_handlers_block_by_func (w, G_CALLBACK (palette_color_notify_cb), profile);
@@ -448,6 +457,14 @@ default_size_reset_cb (GtkWidget *button,
   g_settings_reset (profile, TERMINAL_PROFILE_DEFAULT_SIZE_ROWS_KEY);
 }
 
+static void
+cell_scale_reset_cb (GtkWidget *button,
+                     GSettings *profile)
+{
+  g_settings_reset (profile, TERMINAL_PROFILE_CELL_HEIGHT_SCALE_KEY);
+  g_settings_reset (profile, TERMINAL_PROFILE_CELL_WIDTH_SCALE_KEY);
+}
+
 static void
 reset_compat_defaults_cb (GtkWidget *button,
                           GSettings *profile)
@@ -778,6 +795,14 @@ fixup_color_chooser_button (void)
 
 #endif /* GTK+ < 3.19.8 HACK */
 
+static gboolean
+monospace_filter (const PangoFontFamily *family,
+                  const PangoFontFace   *face,
+                  gpointer data)
+{
+  return pango_font_family_is_monospace ((PangoFontFamily *) family);
+}
+
 /**
  * terminal_profile_edit:
  * @profile: a #GSettings
@@ -850,6 +875,10 @@ terminal_profile_edit (GSettings  *profile,
                     "clicked",
                     G_CALLBACK (default_size_reset_cb),
                     profile);
+  g_signal_connect (gtk_builder_get_object  (builder, "cell-scale-reset-button"),
+                    "clicked",
+                    G_CALLBACK (cell_scale_reset_cb),
+                    profile);
 
   w = (GtkWidget *) gtk_builder_get_object  (builder, "color-scheme-combobox");
   init_color_scheme_menu (w);
@@ -861,7 +890,7 @@ terminal_profile_edit (GSettings  *profile,
       char name[32];
       char *text;
 
-      g_snprintf (name, sizeof (name), "palette-colorpicker-%u", i + 1);
+      g_snprintf (name, sizeof (name), "palette-colorpicker-%u", i);
       w = (GtkWidget *) gtk_builder_get_object  (builder, name);
 
 #if GTK_CHECK_VERSION (3, 19, 8)
@@ -870,11 +899,11 @@ terminal_profile_edit (GSettings  *profile,
 
       g_object_set_data (G_OBJECT (w), "palette-entry-index", GUINT_TO_POINTER (i));
 
-      text = g_strdup_printf (_("Choose Palette Color %u"), i + 1);
+      text = g_strdup_printf (_("Choose Palette Color %u"), i);
       gtk_color_button_set_title (GTK_COLOR_BUTTON (w), text);
       g_free (text);
 
-      text = g_strdup_printf (_("Palette entry %u"), i + 1);
+      text = g_strdup_printf (_("Palette entry %u"), i);
       gtk_widget_set_tooltip_text (w, text);
       g_free (text);
 
@@ -931,15 +960,19 @@ terminal_profile_edit (GSettings  *profile,
                                 (GSettingsBindGetMapping)
                                 string_to_window_title, NULL, NULL, NULL);
 
+  w = GTK_WIDGET (gtk_builder_get_object (builder, "background-colorpicker"));
+#if GTK_CHECK_VERSION (3, 19, 8)
+  g_object_set (w, "show-editor", TRUE, NULL);
+#endif
   g_settings_bind_with_mapping (profile,
                                 TERMINAL_PROFILE_BACKGROUND_COLOR_KEY,
-                                gtk_builder_get_object (builder,
-                                                        "background-colorpicker"),
+                                w,
                                 "rgba",
                                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET,
                                 (GSettingsBindGetMapping) s_to_rgba,
                                 (GSettingsBindSetMapping) rgba_to_s,
                                 NULL, NULL);
+
   g_settings_bind_with_mapping (profile,
                                 TERMINAL_PROFILE_BACKSPACE_BINDING_KEY,
                                 gtk_builder_get_object (builder,
@@ -960,85 +993,115 @@ terminal_profile_edit (GSettings  *profile,
                    G_SETTINGS_BIND_GET |
                    G_SETTINGS_BIND_INVERT_BOOLEAN |
                    G_SETTINGS_BIND_SET);
+
+  w = GTK_WIDGET (gtk_builder_get_object (builder, "bold-colorpicker"));
+#if GTK_CHECK_VERSION (3, 19, 8)
+  g_object_set (w, "show-editor", TRUE, NULL);
+#endif
   g_settings_bind (profile, TERMINAL_PROFILE_BOLD_COLOR_SAME_AS_FG_KEY,
-                   gtk_builder_get_object (builder,
-                                           "bold-colorpicker"),
+                   w,
                    "sensitive",
                    G_SETTINGS_BIND_GET |
                    G_SETTINGS_BIND_INVERT_BOOLEAN |
                    G_SETTINGS_BIND_NO_SENSITIVITY);
   g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_BOLD_COLOR_KEY,
-                                gtk_builder_get_object (builder,
-                                                        "bold-colorpicker"),
+                                w,
                                 "rgba",
                                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
                                 (GSettingsBindGetMapping) s_to_rgba,
                                 (GSettingsBindSetMapping) rgba_to_s,
                                 NULL, NULL);
+
+  g_settings_bind (profile, TERMINAL_PROFILE_CELL_HEIGHT_SCALE_KEY,
+                   gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON
+                                                   (gtk_builder_get_object
+                                                    (builder,
+                                                     "cell-height-scale-spinbutton"))),
+                   "value", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
+  g_settings_bind (profile, TERMINAL_PROFILE_CELL_WIDTH_SCALE_KEY,
+                   gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON
+                                                   (gtk_builder_get_object
+                                                    (builder,
+                                                     "cell-width-scale-spinbutton"))),
+                   "value", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
   g_settings_bind (profile, TERMINAL_PROFILE_CURSOR_COLORS_SET_KEY,
                    gtk_builder_get_object (builder,
                                            "cursor-colors-checkbutton"),
                    "active", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
+
+  w = GTK_WIDGET (gtk_builder_get_object (builder, "cursor-foreground-colorpicker"));
+#if GTK_CHECK_VERSION (3, 19, 8)
+  g_object_set (w, "show-editor", TRUE, NULL);
+#endif
   g_settings_bind (profile, TERMINAL_PROFILE_CURSOR_COLORS_SET_KEY,
-                   gtk_builder_get_object (builder,
-                                           "cursor-foreground-colorpicker"),
-                   "sensitive",
-                   G_SETTINGS_BIND_GET |
-                   G_SETTINGS_BIND_NO_SENSITIVITY);
-  g_settings_bind (profile, TERMINAL_PROFILE_CURSOR_COLORS_SET_KEY,
-                   gtk_builder_get_object (builder,
-                                           "cursor-background-colorpicker"),
+                   w,
                    "sensitive",
                    G_SETTINGS_BIND_GET |
                    G_SETTINGS_BIND_NO_SENSITIVITY);
   g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_CURSOR_FOREGROUND_COLOR_KEY,
-                                gtk_builder_get_object (builder,
-                                                        "cursor-foreground-colorpicker"),
+                                w,
                                 "rgba",
                                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
                                 (GSettingsBindGetMapping) s_to_rgba,
                                 (GSettingsBindSetMapping) rgba_to_s,
                                 NULL, NULL);
+
+  w = GTK_WIDGET (gtk_builder_get_object (builder, "cursor-background-colorpicker"));
+#if GTK_CHECK_VERSION (3, 19, 8)
+  g_object_set (w, "show-editor", TRUE, NULL);
+#endif
+  g_settings_bind (profile, TERMINAL_PROFILE_CURSOR_COLORS_SET_KEY,
+                   w,
+                   "sensitive",
+                   G_SETTINGS_BIND_GET |
+                   G_SETTINGS_BIND_NO_SENSITIVITY);
   g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_CURSOR_BACKGROUND_COLOR_KEY,
-                                gtk_builder_get_object (builder,
-                                                        "cursor-background-colorpicker"),
+                                w,
                                 "rgba",
                                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
                                 (GSettingsBindGetMapping) s_to_rgba,
                                 (GSettingsBindSetMapping) rgba_to_s,
                                 NULL, NULL);
+
   g_settings_bind (profile, TERMINAL_PROFILE_HIGHLIGHT_COLORS_SET_KEY,
                    gtk_builder_get_object (builder,
                                            "highlight-colors-checkbutton"),
                    "active", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
+
+  w = GTK_WIDGET (gtk_builder_get_object (builder, "highlight-foreground-colorpicker"));
+#if GTK_CHECK_VERSION (3, 19, 8)
+  g_object_set (w, "show-editor", TRUE, NULL);
+#endif
   g_settings_bind (profile, TERMINAL_PROFILE_HIGHLIGHT_COLORS_SET_KEY,
-                   gtk_builder_get_object (builder,
-                                           "highlight-foreground-colorpicker"),
-                   "sensitive",
-                   G_SETTINGS_BIND_GET |
-                   G_SETTINGS_BIND_NO_SENSITIVITY);
-  g_settings_bind (profile, TERMINAL_PROFILE_HIGHLIGHT_COLORS_SET_KEY,
-                   gtk_builder_get_object (builder,
-                                           "highlight-background-colorpicker"),
+                   w,
                    "sensitive",
                    G_SETTINGS_BIND_GET |
                    G_SETTINGS_BIND_NO_SENSITIVITY);
   g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_HIGHLIGHT_FOREGROUND_COLOR_KEY,
-                                gtk_builder_get_object (builder,
-                                                        "highlight-foreground-colorpicker"),
+                                w,
                                 "rgba",
                                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
                                 (GSettingsBindGetMapping) s_to_rgba,
                                 (GSettingsBindSetMapping) rgba_to_s,
                                 NULL, NULL);
+
+  w = GTK_WIDGET (gtk_builder_get_object (builder, "highlight-background-colorpicker"));
+#if GTK_CHECK_VERSION (3, 19, 8)
+  g_object_set (w, "show-editor", TRUE, NULL);
+#endif
+  g_settings_bind (profile, TERMINAL_PROFILE_HIGHLIGHT_COLORS_SET_KEY,
+                   w,
+                   "sensitive",
+                   G_SETTINGS_BIND_GET |
+                   G_SETTINGS_BIND_NO_SENSITIVITY);
   g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_HIGHLIGHT_BACKGROUND_COLOR_KEY,
-                                gtk_builder_get_object (builder,
-                                                        "highlight-background-colorpicker"),
+                                w,
                                 "rgba",
                                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
                                 (GSettingsBindGetMapping) s_to_rgba,
                                 (GSettingsBindSetMapping) rgba_to_s,
                                 NULL, NULL);
+
   g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_CURSOR_SHAPE_KEY,
                                 gtk_builder_get_object (builder,
                                                         "cursor-shape-combobox"),
@@ -1047,6 +1110,22 @@ terminal_profile_edit (GSettings  *profile,
                                 (GSettingsBindGetMapping) string_to_enum,
                                 (GSettingsBindSetMapping) enum_to_string,
                                 vte_cursor_shape_get_type, NULL);
+  g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_CURSOR_BLINK_MODE_KEY,
+                                gtk_builder_get_object (builder,
+                                                        "cursor-blink-mode-combobox"),
+                                "active",
+                                G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET,
+                                (GSettingsBindGetMapping) string_to_enum,
+                                (GSettingsBindSetMapping) enum_to_string,
+                                vte_cursor_blink_mode_get_type, NULL);
+  g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_TEXT_BLINK_MODE_KEY,
+                                gtk_builder_get_object (builder,
+                                                        "text-blink-mode-combobox"),
+                                "active",
+                                G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET,
+                                (GSettingsBindGetMapping) string_to_enum,
+                                (GSettingsBindSetMapping) enum_to_string,
+                                vte_text_blink_mode_get_type, NULL);
   g_settings_bind (profile, TERMINAL_PROFILE_CUSTOM_COMMAND_KEY,
                    gtk_builder_get_object (builder, "custom-command-entry"),
                    "text", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
@@ -1078,18 +1157,25 @@ terminal_profile_edit (GSettings  *profile,
                                 (GSettingsBindGetMapping) string_to_enum,
                                 (GSettingsBindSetMapping) enum_to_string,
                                 terminal_exit_action_get_type, NULL);
+  w = (GtkWidget*) gtk_builder_get_object (builder, "font-selector");
+  gtk_font_chooser_set_filter_func (GTK_FONT_CHOOSER (w), monospace_filter, NULL, NULL);
   g_settings_bind (profile, TERMINAL_PROFILE_FONT_KEY,
-                   gtk_builder_get_object (builder, "font-selector"),
+                   w,
                    "font-name", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
+
+  w = GTK_WIDGET (gtk_builder_get_object (builder, "foreground-colorpicker"));
+#if GTK_CHECK_VERSION (3, 19, 8)
+      g_object_set (w, "show-editor", TRUE, NULL);
+#endif
   g_settings_bind_with_mapping (profile,
                                 TERMINAL_PROFILE_FOREGROUND_COLOR_KEY,
-                                gtk_builder_get_object (builder,
-                                                        "foreground-colorpicker"),
+                                w,
                                 "rgba",
                                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET,
                                 (GSettingsBindGetMapping) s_to_rgba,
                                 (GSettingsBindSetMapping) rgba_to_s,
                                 NULL, NULL);
+
   g_settings_bind (profile, TERMINAL_PROFILE_LOGIN_SHELL_KEY,
                    gtk_builder_get_object (builder,
                                            "login-shell-checkbutton"),
